@@ -1,8 +1,6 @@
 /*
    +----------------------------------------------------------------------+
-   | PHP Version 5                                                        |
-   +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2013 The PHP Group                                |
+   | Copyright (c) The PHP Group                                          |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -16,11 +14,15 @@
    +----------------------------------------------------------------------+
  */
 
-/* $Id$ */
-
 #define ZEND_INCLUDE_FULL_WINDOWS_HEADERS
 
 #include "php.h"
+#ifdef strcasecmp
+# undef strcasecmp
+#endif
+#ifdef strncasecmp
+# undef strncasecmp
+#endif
 #include "php_ini.h"
 #include "php_apache.h"
 
@@ -60,17 +62,17 @@ static const char *real_value_hnd(cmd_parms *cmd, void *dummy, const char *name,
 	php_dir_entry e;
 
 	phpapdebug((stderr, "Getting %s=%s for %p (%d)\n", name, value, dummy, zend_hash_num_elements(&d->config)));
-	
+
 	if (!strncasecmp(value, "none", sizeof("none"))) {
 		value = "";
 	}
-	
+
 	e.value = apr_pstrdup(cmd->pool, value);
 	e.value_len = strlen(value);
 	e.status = status;
 	e.htaccess = ((cmd->override & (RSRC_CONF|ACCESS_CONF)) == 0);
 
-	zend_hash_update(&d->config, (char *) name, strlen(name) + 1, &e, sizeof(e), NULL);
+	zend_hash_str_update_mem(&d->config, (char *) name, strlen(name), &e, sizeof(e));
 	return NULL;
 }
 
@@ -117,55 +119,61 @@ static const char *php_apache_phpini_set(cmd_parms *cmd, void *mconfig, const ch
 	return NULL;
 }
 
-static zend_bool should_overwrite_per_dir_entry(HashTable *target_ht, php_dir_entry *new_per_dir_entry, zend_hash_key *hash_key, void *pData)
+static zend_bool should_overwrite_per_dir_entry(HashTable *target_ht, zval *zv, zend_hash_key *hash_key, void *pData)
 {
+	php_dir_entry *new_per_dir_entry = Z_PTR_P(zv);
 	php_dir_entry *orig_per_dir_entry;
 
-	if (zend_hash_find(target_ht, hash_key->arKey, hash_key->nKeyLength, (void **) &orig_per_dir_entry)==FAILURE) {
+	if ((orig_per_dir_entry = zend_hash_find_ptr(target_ht, hash_key->key)) == NULL) {
 		return 1; /* does not exist in dest, copy from source */
 	}
 
 	if (new_per_dir_entry->status >= orig_per_dir_entry->status) {
 		/* use new entry */
-		phpapdebug((stderr, "ADDING/OVERWRITING %s (%d vs. %d)\n", hash_key->arKey, new_per_dir_entry->status, orig_per_dir_entry->status));
+		phpapdebug((stderr, "ADDING/OVERWRITING %s (%d vs. %d)\n", ZSTR_VAL(hash_key->key), new_per_dir_entry->status, orig_per_dir_entry->status));
 		return 1;
 	} else {
 		return 0;
 	}
 }
 
+void config_entry_ctor(zval *zv)
+{
+	php_dir_entry *pe = (php_dir_entry*)Z_PTR_P(zv);
+	php_dir_entry *npe = malloc(sizeof(php_dir_entry));
+
+	memcpy(npe, pe, sizeof(php_dir_entry));
+	ZVAL_PTR(zv, npe);
+}
 
 void *merge_php_config(apr_pool_t *p, void *base_conf, void *new_conf)
 {
 	php_conf_rec *d = base_conf, *e = new_conf, *n = NULL;
-#if STAS_0
-	php_dir_entry *pe;
-	php_dir_entry *data;
-	char *str;
-	uint str_len;
-	ulong num_index;
+#ifdef ZTS
+	zend_string *str;
+	zval *data;
 #endif
 
 	n = create_php_config(p, "merge_php_config");
 	/* copy old config */
-	zend_hash_copy(&n->config, &d->config, NULL, NULL, sizeof(php_dir_entry));
+#ifdef ZTS
+	ZEND_HASH_FOREACH_STR_KEY_VAL(&d->config, str, data) {
+		zend_string *key;
+		zval *new_entry;
+
+		/* Avoid sharing the non interned string among threads. */
+		key = zend_string_dup(str, 1);
+
+		new_entry = zend_hash_add(&n->config, key, data);
+
+		config_entry_ctor(new_entry);
+	} ZEND_HASH_FOREACH_END();
+#else
+	zend_hash_copy(&n->config, &d->config, config_entry_ctor);
+#endif
 	/* merge new config */
 	phpapdebug((stderr, "Merge dir (%p)+(%p)=(%p)\n", base_conf, new_conf, n));
-	zend_hash_merge_ex(&n->config, &e->config, NULL, sizeof(php_dir_entry), (merge_checker_func_t) should_overwrite_per_dir_entry, NULL);
-#if STAS_0
-	for (zend_hash_internal_pointer_reset(&d->config);
-			zend_hash_get_current_key_ex(&d->config, &str, &str_len, 
-				&num_index, 0, NULL) == HASH_KEY_IS_STRING;
-			zend_hash_move_forward(&d->config)) {
-		pe = NULL;
-		zend_hash_get_current_data(&d->config, (void **) &data);
-		if (zend_hash_find(&n->config, str, str_len, (void **) &pe) == SUCCESS) {
-			if (pe->status >= data->status) continue;
-		}
-		phpapdebug((stderr, "ADDING/OVERWRITING %s (%d vs. %d)\n", str, data->status, pe?pe->status:-1));
-		zend_hash_update(&n->config, str, str_len, data, sizeof(*data), NULL);
-	}
-#endif
+	zend_hash_merge_ex(&n->config, &e->config, config_entry_ctor, should_overwrite_per_dir_entry, NULL);
 	return n;
 }
 
@@ -173,8 +181,8 @@ char *get_php_config(void *conf, char *name, size_t name_len)
 {
 	php_conf_rec *d = conf;
 	php_dir_entry *pe;
-	
-	if (zend_hash_find(&d->config, name, name_len, (void **) &pe) == SUCCESS) {
+
+	if ((pe = zend_hash_str_find_ptr(&d->config, name, name_len)) != NULL) {
 		return pe->value;
 	}
 
@@ -184,21 +192,15 @@ char *get_php_config(void *conf, char *name, size_t name_len)
 void apply_config(void *dummy)
 {
 	php_conf_rec *d = dummy;
-	char *str;
-	uint str_len;
+	zend_string *str;
 	php_dir_entry *data;
-	
-	for (zend_hash_internal_pointer_reset(&d->config);
-			zend_hash_get_current_key_ex(&d->config, &str, &str_len, NULL, 0, 
-				NULL) == HASH_KEY_IS_STRING;
-			zend_hash_move_forward(&d->config)) {
-		if (zend_hash_get_current_data(&d->config, (void **) &data) == SUCCESS) {
-			phpapdebug((stderr, "APPLYING (%s)(%s)\n", str, data->value));
-			if (zend_alter_ini_entry(str, str_len, data->value, data->value_len, data->status, data->htaccess?PHP_INI_STAGE_HTACCESS:PHP_INI_STAGE_ACTIVATE) == FAILURE) {
-				phpapdebug((stderr, "..FAILED\n"));
-			}
+
+	ZEND_HASH_FOREACH_STR_KEY_PTR(&d->config, str, data) {
+		phpapdebug((stderr, "APPLYING (%s)(%s)\n", ZSTR_VAL(str), data->value));
+		if (zend_alter_ini_entry_chars(str, data->value, data->value_len, data->status, data->htaccess?PHP_INI_STAGE_HTACCESS:PHP_INI_STAGE_ACTIVATE) == FAILURE) {
+			phpapdebug((stderr, "..FAILED\n"));
 		}
-	}
+	} ZEND_HASH_FOREACH_END();
 }
 
 const command_rec php_dir_cmds[] =
@@ -215,27 +217,23 @@ static apr_status_t destroy_php_config(void *data)
 {
 	php_conf_rec *d = data;
 
-	phpapdebug((stderr, "Destroying config %p\n", data));	
+	phpapdebug((stderr, "Destroying config %p\n", data));
 	zend_hash_destroy(&d->config);
 
 	return APR_SUCCESS;
 }
 
+static void config_entry_dtor(zval *zv)
+{
+	free((php_dir_entry*)Z_PTR_P(zv));
+}
+
 void *create_php_config(apr_pool_t *p, char *dummy)
 {
 	php_conf_rec *newx = (php_conf_rec *) apr_pcalloc(p, sizeof(*newx));
 
 	phpapdebug((stderr, "Creating new config (%p) for %s\n", newx, dummy));
-	zend_hash_init(&newx->config, 0, NULL, NULL, 1);
+	zend_hash_init(&newx->config, 0, NULL, config_entry_dtor, 1);
 	apr_pool_cleanup_register(p, newx, destroy_php_config, apr_pool_cleanup_null);
 	return (void *) newx;
 }
-
-/*
- * Local variables:
- * tab-width: 4
- * c-basic-offset: 4
- * End:
- * vim600: sw=4 ts=4 fdm=marker
- * vim<600: sw=4 ts=4
- */
